
;**************************************************************************
;*        LCD hodiny s RTC DS1306                                         *
;*        LCD driver: ICM7211AM                                           *
;**************************************************************************
;**************************************************************************
;*        v 1.08 - 12.04.2020                                             *
;**************************************************************************
;*        Hardw.: PIC16F1828 used                                         *
;*        OSC.......: Internal osc. used     POWER.....:   5,0V extern.   *
;**************************************************************************

 INCLUDE "P16F1828.INC"

_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002

;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
;--------------------------------------------------------------------------
v_digit4	EQU	H'22'
v_digit3	EQU	H'23'
v_digit2	EQU	H'24'
v_digit1	EQU	H'25'

v_rtc_sec	EQU	H'26'
v_rtc_min	EQU	H'27'
v_rtc_hrs	EQU	H'28'

v_msb		EQU	H'29'
v_lsb		EQU	H'2A'

v_value		EQU	H'2B'
v_spi_buffi	EQU	H'2C'
v_digit1_src	EQU	H'2D'
;--------------------------------------------------------------------------
;==========================================================================
		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;              ++  RESET :  main boot routine  ++
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
init_main	nop
		clrf	BSR
		clrf	INTCON			;disable interrupts

		banksel	OPTION_REG
		movlw	B'00111111'
		movwf	OPTION_REG		;see datasheet Page 176
						;bit 7 = 0 --> Week pull-ups are enabled
		movlw	H'03'
		movwf	BSR			;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		banksel	TRISA
		movlw	B'11111111'
		movwf	TRISA

		movlw	B'00000000'		;7-segment dirvers
		movwf	TRISC

		movlw	B'00001111'
		movwf	TRISB

		movf	OSCCON,0
		andlw	B'11111111'
		iorlw	B'01111000'
		movwf	OSCCON			;set 16MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		banksel	WPUA
		movlw	B'00111111'
		movwf	WPUA
		
		clrf	BSR			;select BANK 0

		call	spi_init_m1		;MSSP1 as SPI master in mode1 initialization

		call	ds1306_tce
		goto	main_prg
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Program
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
main_prg	clrf	BSR

		movlw	H'FF'
		movwf	PORTC

		bsf	LATB,D'007'
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
main_loop	nop
		call	rtc_spi_read		;(1) - read the time information from ISL12022
		call	translate_clk
		call	transl_h1_no
		call	icm7211_store
		call	ml_delay

		call	rtc_spi_read		;(1) - read the time information from ISL12022
		call	translate_clk
		call	transl_h1_dp
		call	icm7211_store
		call	ml_delay

		btfss	PORTA,D'000'
		goto	clk_set_up

		goto	main_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
clk_set_up	nop
		movlw	H'00'
		movwf	v_rtc_sec
		call	dr2
;-------------------------------------
clk_su_loop	btfss	PORTA,D'001'
		call	inc_hrs
		btfss	PORTA,D'002'
		call	inc_min
;display the clock -----------------------------
		call	translate_clk
		call	transl_h1_no
		call	icm7211_store
;-----------------------------------------------
		call	dr4
;
		btfss	PORTA,D'000'
		goto	clk_su_loop
		call	rtc_spi_write
;-------------------------------------
		goto	main_loop
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
inc_hrs		incf	v_rtc_hrs,1
		movf	v_rtc_hrs,0
		xorlw	H'24'
		btfsc	STATUS,Z
		clrf	v_rtc_hrs

		movf	v_rtc_hrs,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfss	STATUS,Z
		return
		movf	v_rtc_hrs,0
		andlw	B'11110000'
		addlw	H'10'
		movwf	v_rtc_hrs
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
inc_min		incf	v_rtc_min,1
		movf	v_rtc_min,0
		xorlw	H'5A'
		btfsc	STATUS,Z
		clrf	v_rtc_min

		movf	v_rtc_min,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfss	STATUS,Z
		return
		movf	v_rtc_min,0
		andlw	B'11110000'
		addlw	H'10'
		movwf	v_rtc_min
		return
;------------------------------------------------------------------------------
clear_hrs	clrf	v_rtc_hrs
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
transl_h1_no	movf	v_digit1_src,0
		movwf	v_digit1
		btfss	STATUS,Z
		return
		movlw	H'0A'
		movwf	v_digit1
		return
;------------------------------------------------------------------------------
transl_h1_dp	movf	v_digit1_src,0
		btfsc	STATUS,Z
		goto	h1_0dp
		movf	v_digit1_src,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	h1_1dp
;------------------------------------------------
h1_2dp		movlw	H'0E'
		movwf	v_digit1
		return
;------------------------------------------------
h1_0dp		movlw	H'0D'
		movwf	v_digit1
		return
;------------------------------------------------
h1_1dp		movlw	H'04'
		movwf	v_digit1
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
icm7211_store	call	icm7211_store1
		call	icm7211_store2
		call	icm7211_store3
		call	icm7211_store4
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
icm7211_store1	movf	v_digit1,0
		addlw	B'11110000'
		movwf	LATC

		bsf	LATC,D'004'	;DS1 = 1
		bsf	LATC,D'005'	;DS2 = 1
		goto	icm_7211_strobe
;------------------------------------------------------------------------------
icm7211_store2	movf	v_digit2,0
		addlw	B'11110000'
		movwf	LATC

		bcf	LATC,D'004'	;DS1 = 0
		bsf	LATC,D'005'	;DS2 = 1
		goto	icm_7211_strobe
;------------------------------------------------------------------------------
icm7211_store3	movf	v_digit3,0
		addlw	B'11110000'
		movwf	LATC

		bsf	LATC,D'004'	;DS1 = 1
		bcf	LATC,D'005'	;DS2 = 0
		goto	icm_7211_strobe
;------------------------------------------------------------------------------
icm7211_store4	movf	v_digit4,0
		addlw	B'11110000'
		movwf	LATC

		bcf	LATC,D'004'	;DS1 = 0
		bcf	LATC,D'005'	;DS2 = 0
		goto	icm_7211_strobe
;------------------------------------------------------------------------------
icm_7211_strobe	bcf	LATC,D'006'	;CS1 = 0
		bcf	LATB,D'007'	;CS2 = 0
		nop
		bsf	LATB,D'007'	;CS2 = 1
		bsf	LATC,D'006'	;CS1 = 0
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ml_delay	call	dr2
		call	dr2
		call	dr2
		call	dr2
		return
;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
rtc_spi_read	call	spi_cs0_en		;(1) - SPI communication start

		movlw	H'00'
		call	spi_send		;(2)
;
		call	spi_send		;(3.1)
		movwf	v_rtc_sec
;
		call	spi_send		;(3.2)
		movwf	v_rtc_min
;
		call	spi_send		;(3.3)
		movwf	v_rtc_hrs
;
		call	spi_cs0_dis		;(4)
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
translate_clk	movf	v_rtc_hrs,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	no_clk
;------------------------------------------
		movf	v_rtc_hrs,0
		call	calc_hex_disp
		movf	v_msb,0
		movwf	v_digit1_src
		movf	v_lsb,0
		movwf	v_digit2
;------------------------------------------
		movf	v_rtc_min,0
		call	calc_hex_disp
		movf	v_msb,0
		movwf	v_digit3
		movf	v_lsb,0
		movwf	v_digit4

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
no_clk		movlw	H'0A'
		movwf	v_digit4
		movwf	v_digit3
		movwf	v_digit2

		movlw	H'00'
		movwf	v_digit1_src

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
calc_hex_disp	movwf	v_value
		andlw	B'00001111'
		movwf	v_lsb
		movf	v_value,0
		movwf	v_msb
		lsrf	v_msb,1
		lsrf	v_msb,1
		lsrf	v_msb,1
		lsrf	v_msb,1
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;DS1306 - Trickle Charger Enable
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ds1306_tce	call	spi_cs0_en		;(1) - SPI communication start

		movlw	H'91'			;Trickle-Charge Register
		call	spi_send		;(2)

		movlw	B'10100101'
		call	spi_send

		call	spi_cs0_dis		;(4)
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_spi_write	nop
		call	spi_cs0_en		;(1) - SPI communication start
;
		movlw	H'80'
		call	spi_send		;(2)
;
		movf	v_rtc_sec,0
		call	spi_send		;(3.1)
;
		movf	v_rtc_min,0
		call	spi_send		;(3.2)
;
		movf	v_rtc_hrs,0
		call	spi_send		;(3.3)

		call	spi_cs0_dis		;(4)
		return
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
 INCLUDE "91_SPI_comm_16F1828.inc"
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
		END